--[[ 
	Shadowed Unit Frames (Deficit), largely based off source from ShadowedUF Bars
]]

local Deficit = select(2, ...)
local L = Deficit.L
local SL = ShadowUF.L or ShadowUFLocals
ShadowUF:RegisterModule(Deficit, "defbars")

function Deficit:OnConfigurationLoad()
	for _, unit in pairs(ShadowUF.unitList) do
		local data = ShadowUF.db.profile.units[unit]
		data.healthBar.deficit = data.healthBar.deficit or false
		data.powerBar.deficit = data.powerBar.deficit or false
		data.altPowerBar.deficit = data.altPowerBar.deficit or false
	end
	
	local barTable = ShadowUF.Config.barTable

	barTable.args.deficitOpt = {
		order = 2.1,
		type = "toggle",
		name = L["Deficit"],
		desc = L["Display this bar as a deficit so that at full value, the bar will be empty and at empty value, the bar will be full"],
		arg = "$parent.deficit",
		hidden = function(info) return info[#(info) - 1] ~= "healthBar" and info[#(info) - 1] ~= "powerBar" and info[#(info) - 1] ~= "altPowerBar" end,
	}
end

-- Hook the OnEnables to ensure we have the first hook on the bars
local function hookModule(module)
	local OnEnable = module.OnEnable
	module.OnEnable = function(self, frame, ...)
		OnEnable(self, frame, ...)

		local bar = frame[self.moduleKey]
		if( not bar.defFlag ) then
			bar.defFlag = true
			
			local oldSetValue = bar.SetValue
			local optTable = ShadowUF.db.profile.units[bar.parent.unitType][self.moduleKey]
			bar.SetValue = function(self, value)
				if ( optTable.deficit ) then
					local min, max = self:GetMinMaxValues()
					value = max - value
					-- value = value > self.maxValue and self.maxValue or value
				end
				return oldSetValue(self, value)
			end
		end
	end
end

hookModule(ShadowUF.modules.healthBar)
hookModule(ShadowUF.modules.powerBar)
hookModule(ShadowUF.modules.altPowerBar)

for frame in pairs(ShadowUF.Units.frameList) do
	if( frame.visibility.healthBar ) then
		ShadowUF.modules.healthBar:OnEnable(frame)

		if( frame.visibility.incHeal ) then
			ShadowUF.modules.incHeal:OnEnable(frame)
		end
	end
	
	if( frame.visibility.powerBar ) then
		ShadowUF.modules.powerBar:OnEnable(frame)
	end
	
	if( frame.visibility.altPowerBar ) then
		ShadowUF.modules.altPowerBar:OnEnable(frame)
	end
	
	if( frame.visibility.healthBar or frame.visibility.powerBar or frame.visibility.altPowerBar ) then
		ShadowUF.Layout:Load(frame)
		ShadowUF:FireModuleEvent("OnLayoutReload", frame.unit)
		frame:FullUpdate()
	end
end
